import * as math from 'mathjs';
import { Equation, parse } from 'algebra.js';
import { HttpStatus, UnprocessableEntityException } from '@nestjs/common';

export function calculateSalesFormula(
  formula: string,
  taxes: object[],
  total: number,
  throwError = false,
) {
  try {
    let { value, variable, tax_datas } = mappingTaxes(taxes, formula, total);

    const x1 = math.simplify(formula, variable).toString();
    console.log('Formula ', x1);
    const dppFormula = parse(x1);
    const totalFormula = parse(total.toString());
    const equation = new Equation(totalFormula, dppFormula);

    console.log(equation.toString());
    const result = equation.solveFor('dpp').toString();
    console.log(result, 'formula');

    value = math.evaluate(result);
    console.log(value, 'value');

    return {
      dpp_value: value,
      tax_datas: tax_datas,
    };
  } catch (e) {
    returnError(throwError, e, taxes);
  }
}

export function calculateProfitFormula(
  formula: string,
  taxes: object[],
  total: number,
  profit_share = 0,
  throwError = false,
) {
  try {
    let { value, variable, tax_datas } = mappingTaxes(
      taxes,
      formula,
      total,
      profit_share,
    );

    const result = math.simplify(formula, variable).toString();
    console.log(result, 'formula');

    value = math.evaluate(result);
    console.log(value, 'value');

    return {
      dpp_value: value,
      tax_datas: tax_datas,
    };
  } catch (e) {
    returnError(throwError, e, taxes);
  }
}

function mappingTaxes(taxes, formula, total, profit_share = 0) {
  let value = 0;
  const variable = {};
  let tax_datas = [];
  const const_variable = ['profit_share', 'item_share', 'dpp'];

  const regex = /([a-zA-Z0-9_]+)/g;

  const matches: string[] = formula.match(regex);
  const uniqueMatches = new Set(matches);
  const keys = Array.from(uniqueMatches);

  for (const key of keys) {
    if (!const_variable.includes(key)) {
      const keyData = taxes.find((tax) => tax.name == key);
      variable[key] = keyData.value / 100;

      tax_datas.push({
        tax_id: keyData.id,
        tax_name: keyData.name,
        tax_value: keyData.value,
        tax_total_value: (keyData.value / 100) * Number(total),
      });
    } else {
      switch (key) {
        case 'profit_share':
          variable[key] = profit_share / 100;
          break;

        case 'item_share':
          variable[key] = profit_share / 100;
          break;

        case 'dpp':
          if (profit_share > 0) variable[key] = total;
          break;
        default:
          variable[key] = profit_share;
          break;
      }
    }
  }

  return {
    value: value,
    variable: variable,
    tax_datas: tax_datas,
  };
}

function returnError(throwError, e, taxes) {
  if (throwError) {
    throw new UnprocessableEntityException({
      statusCode: HttpStatus.UNPROCESSABLE_ENTITY,
      message: `Gagal! Formula error`,
      error: 'Unprocessable Entity',
    });
  } else {
    console.log(e);

    return {
      dpp_value: 0,
      tax_datas: taxes,
    };
  }
}
